#!/usr/bin/R

##############################################################################
#
# MODULE:       r.randomwalk.map.r
#
# AUTHOR:       Martin Mergili
#
# PURPOSE:      Conceptual tool for mass movement propagation
#               Script for map plots
#
# COPYRIGHT:    (c) 2013 - 2016 by the author
#               (c) 2013 - 2016 by the BOKU University, Vienna
#               (c) 2015 - 2016 by the University of Vienna
#               (C) 1993 - 2016 by the R Development Core Team
#
# VERSION:      20160802 (2 August 2016)
#
#               This program is free software under the GNU General Public
#               License (>=v2).
#
##############################################################################

#Loading libraries
library(sp)
library(rgdal)
library(raster)

#Importing arguments (defined in r.randomwalk.py)
args<-commandArgs(trailingOnly = TRUE)
temppath<-args[1]
filepath<-args[2]
prefix<-args[3]
yrmax<-as.numeric(args[4])
yrmin<-as.numeric(args[5])
xrmin<-as.numeric(args[6])
xrmax<-as.numeric(args[7])
csz<-as.numeric(args[8])
map<-as.character(args[9])
numplot<-as.integer(args[10])
typeext<-as.integer(args[11])
gctrl<-as.integer(args[12])
colctrl<-as.integer(args[13])
mode<-as.integer(args[14])
ictrl<-as.integer(args[15])
lctrl<-as.integer(args[16])
tctrl<-as.integer(args[17])
impactdef<-as.integer(args[18])
releasedef<-as.integer(args[19])
releasefile<-args[20]
depositdef<-as.integer(args[21])
mflag<-as.integer(args[22])
mapprof<-as.integer(args[23])

ctrl_thresholds=2

#Defining names of raster maps
hillshmap<-paste(temppath,'/hillshade.asc', sep='')
layermap<-paste(temppath,'/',map,'.asc', sep='')

#Importing and preprocessing raster maps
hillshade<-raster(hillshmap)
layer<-raster(layermap)
layer[layer<0]<-NA
layer[is.na(layer)==TRUE]<-NA

#Defining threshold levels, colours and labels
lev<-vector('numeric', 6)
lab<-vector('expression', 7)
lev00<--1
col0<-rgb(0.0,0.0,0.0,0.00)

if ( mode==0 ) { #impact frequency 

  titlong<-'Impact frequency'
  titshort<-substitute(expression(~~~~~~~~~~~IF))

  lev[1]<-0
  layer[layer==lev[1]]<-NA

  qlev<-quantile(layer, c(0.0,0.2,0.4,0.6,0.8,0.9,1.0)) #threshold levels

  layer2<-layer
  for (i in 2:7) {
    layer2[layer>qlev[i-1]]<-i-1
    lev[i]<-i-1
  }
  layer<-layer2

  lab[1]<-'Q0' #legend labels
  lab[2]<-'Q20'
  lab[3]<-'Q40'
  lab[4]<-'Q60'
  lab[5]<-'Q80'
  lab[6]<-'Q90'
  lab[7]<-'Q100'

  col1<-rgb(0.8,0.5,0.0,0.4) #colours
  col2<-rgb(0.6,0.3,0.3,0.5)
  col3<-rgb(0.1,0.5,0.5,0.6)
  col4<-rgb(0.1,0.2,0.9,0.7)
  col5<-rgb(0.3,0.1,0.5,0.8)
  col6<-rgb(0.8,0.1,0.2,0.9)

  coll<-c(col1,col2,col3,col4,col5,col6)
  levl<-lev

} else if ( mode==1 || mode==2 ) { #impact scores 

  if ( mode == 1 && mflag == 0 ) {
    titlong<-'Impact indicator score'
    titshort<-expression(~~~~~~~~~~~IIS)

  } else if ( mode == 1 && mflag == 1 ) {
    titlong<-'Impact indicator index'
    titshort<-expression(~~~~~~~~~~~III)

  } else {
    titlong<-'Impact hazard indicator score'
    titshort<-expression(~~~~~~~~~~~IHIS)
  }

  lev[1]<-0
  layer[layer==lev[1]]<-NA

  if ( mflag==0 ) {
    for (i in 2:7) { #threshold levels
      lev[i]<-i-1
    }

    lab[1]<-'L' #legend labels
    lab[2]<-''
    lab[3]<-''
    lab[4]<-''
    lab[5]<-''
    lab[6]<-''
    lab[7]<-''
    lab[8]<-'H'

  } else {

    rd<-3 #precision of legend labels

    if ( ctrl_thresholds==1 ) {

      qlev<-quantile(layer, c(0,1/6,2/6,3/6,4/6,5/6,1)) #threshold levels and legend labels
      if ( qlev[7]==qlev[6] ) {
        qlev[6]=(qlev[5]+qlev[7])/2
      }
    } else {

      qlev<-vector('numeric', 7)
      qlev[1]=0.0
      qlev[2]=0.1
      qlev[3]=0.2
      qlev[4]=0.4
      qlev[5]=0.6
      qlev[6]=0.8
      qlev[7]=1.0
    }

    lab[1]<-'0.000'
    layer2<-layer
    layer2[layer>0]<-1
    for (i in 2:7) {
      layer2[layer>qlev[i]]<-i
      lab[i]<-format(round(qlev[i],rd),nsmall=rd)
      lev[i]<-i-1
    }
    layer<-layer2
  }

  col1<-rgb(0.8,0.5,0.0,0.4) #colours
  col2<-rgb(0.6,0.3,0.3,0.5)
  col3<-rgb(0.1,0.5,0.5,0.6)
  col4<-rgb(0.1,0.2,0.9,0.7)
  col5<-rgb(0.3,0.1,0.5,0.8)
  col6<-rgb(0.8,0.1,0.2,0.9)

  if ( mflag==0 ) {
    coll<-c(col0,col1,col2,col3,col4,col5,col6)
    levl<-c(lev00,lev)
  } else {
    coll<-c(col1,col2,col3,col4,col5,col6)
    levl<-lev
  }

} else if ( mode==3 ) { #release zone

  titlong<-'Area of possible release zone'
  titshort<-expression(~~~~~~~~~~~italic(A)[RZ]~(km^2))

  rd<-3 #precision of legend labels

  lev[1]<-0
  layer[layer==lev[1]]<-NA

  qlev<-quantile(layer, c(0,1/6,2/6,3/6,4/6,5/6,1)) #threshold levels and legend labels
  if ( qlev[7]==qlev[6] ) {
    qlev[6]=(qlev[5]+qlev[7])/2
  }

  lab[1]<-'0.000'
  layer2<-layer
  layer2[layer>0]<-1
  for (i in 2:7) {
    layer2[layer>qlev[i]]<-i
    lab[i]<-format(round(qlev[i]/1000000,rd),nsmall=rd)
    lev[i]<-i-1
  }
  layer<-layer2

  col1<-rgb(0.8,0.5,0.0,0.4) #colours
  col2<-rgb(0.6,0.3,0.3,0.5)
  col3<-rgb(0.1,0.5,0.5,0.6)
  col4<-rgb(0.1,0.2,0.9,0.7)
  col5<-rgb(0.3,0.1,0.5,0.8)
  col6<-rgb(0.8,0.1,0.2,0.9)

  coll<-c(col1,col2,col3,col4,col5,col6)
  levl<-lev

} else if ( mode==4 || mode==5 || mode==6 ) { #probabilities 

  if ( mode==4 ) {
    titlong<-'Zonal release probability'
    titshort<-substitute(expression(~~~~~~~~~~~italic(P)[RZ]))

  } else if ( mode==5 ) {
    titlong<-'Impact probability'
    titshort<-substitute(expression(~~~~~~~~~~~italic(P)[I]))

  } else if ( mode==6 ) {
    titlong<-'Composite impact probability'
    titshort<-substitute(expression(~~~~~~~~~~~italic(P)[IC]))
  }

  rd<-3 #precision of legend labels
  ctrl_thresholds<-1

  lev[1]<-0
  layer[layer==lev[1]]<-NA

  if ( ctrl_thresholds==1 ) {

    qlev<-quantile(layer, c(0,1/6,2/6,3/6,4/6,5/6,1)) #threshold levels and legend labels
    if ( qlev[7]==qlev[6] ) {
      qlev[6]=(qlev[5]+qlev[7])/2
    }
  } else {

    qlev<-vector('numeric', 7)
    qlev[1]=0.0
    qlev[2]=0.1
    qlev[3]=0.2
    qlev[4]=0.4
    qlev[5]=0.6
    qlev[6]=0.8
    qlev[7]=1.0
  }

  lab[1]<-'0.000'
  layer2<-layer
  layer2[layer>0]<-1
  for (i in 2:7) {
    layer2[layer>qlev[i]]<-i
    lab[i]<-format(round(qlev[i],rd),nsmall=rd)
    lev[i]<-i-1
  }
  layer<-layer2

  col1<-rgb(0.8,0.5,0.0,0.4) #colours
  col2<-rgb(0.6,0.3,0.3,0.5)
  col3<-rgb(0.1,0.5,0.5,0.6)
  col4<-rgb(0.1,0.2,0.9,0.7)
  col5<-rgb(0.3,0.1,0.5,0.8)
  col6<-rgb(0.8,0.1,0.2,0.9)

  coll<-c(col1,col2,col3,col4,col5,col6)
  levl<-lev

} else if ( mode==7 || mode==8 ) { #probabilities 

  if ( mode==7 ) {
    titlong<-'Stddev. of zonal release probability'
    titshort<-substitute(expression(~~~~~~~~~~~italic(sigma)[PRZ]))

  } else if ( mode==8 ) {
    titlong<-'Stddev. of impact probability'
    titshort<-substitute(expression(~~~~~~~~~~~italic(sigma)[PI]))
  }

  rd<-3 #precision of legend labels

  lev[1]<-0
  layer[layer==lev[1]]<-NA

  qlev<-quantile(layer, c(0,1/6,2/6,3/6,4/6,5/6,1)) #threshold levels and legend labels
  if ( qlev[7]==qlev[6] ) {
    qlev[6]=(qlev[5]+qlev[7])/2
  }

  lab[1]<-'0.000'
  layer2<-layer
  layer2[layer>0]<-1
  for (i in 2:7) {
    layer2[layer>qlev[i]]<-i
    lab[i]<-format(round(qlev[i],rd),nsmall=rd)
    lev[i]<-i-1
  }
  layer<-layer2

  col1<-rgb(0.2,0.7,0.2,0.4) #colours
  col2<-rgb(0.2,0.5,0.4,0.5)
  col3<-rgb(0.1,0.3,0.7,0.6)
  col4<-rgb(0.1,0.0,0.9,0.7)
  col5<-rgb(0.5,0.1,0.5,0.8)
  col6<-rgb(0.8,0.4,0.2,0.9)

  coll<-c(col1,col2,col3,col4,col5,col6)
  levl<-lev
}

#Reading and formatting release point data
if ( releasefile!='0' ) {

  pointdata<-read.table(releasefile, header=TRUE)

  pointdata$COLOUR<-'0'
  pointdata$COLOUR[pointdata$RIS==-9999]<-col0
  pointdata$COLOUR[pointdata$RIS==0]<-col0
  pointdata$COLOUR[pointdata$RIS==1]<-col1
  pointdata$COLOUR[pointdata$RIS==2]<-col2
  pointdata$COLOUR[pointdata$RIS==3]<-col3
  pointdata$COLOUR[pointdata$RIS==4]<-col4
  pointdata$COLOUR[pointdata$RIS==5]<-col5
  pointdata$COLOUR[pointdata$RIS==6]<-col6
  nrows<-nrow(pointdata) #number of lines
}

#Computing x and y extent of raster layers 
xdiff<-xrmax-xrmin
ydiff<-yrmax-yrmin

#Adapting boundaries for vector layers
xsmin<-xrmin+0.035*xdiff
xsmax<-xrmax-0.035*xdiff
ysmin<-yrmin+0.035*ydiff
ysmax<-yrmax-0.035*ydiff

#Layouting map
asprat<-xdiff/(1.4*ydiff)
margx<-6.0
margy<-2.5
dispx<-15
dispy<-(15-margx)/asprat+margy

#Importing vector maps
if ( impactdef==1 ) {
  impactshape<-readOGR(temppath, layer='v_limpactmap')
}
if ( releasedef==1 ) {
  releaseshape<-readOGR(temppath, layer='v_lreleasemap')
}
if ( depositdef==1 ) {
  depositshape<-readOGR(temppath, layer='v_ldepositmap')
}

#Creating plot file
if ( typeext==0 ) {
  if ( colctrl==1 ) {
    mapplot<-paste(prefix,'_results/',prefix,'_plots/',prefix,'_hmap_',map,'.png', sep='')
  } else {
    mapplot<-paste(prefix,'_results/',prefix,'_plots/',prefix,'_map_',map,'.png', sep='')
  }
}
png(filename=mapplot, width=dispx, height=dispy, units='cm', res=300)

#Defining margins
par(mar=c(2.0,2.0,0.5,1.0))

#Plotting hillshade
if ( colctrl==1 ) {
  plot(hillshade, alpha=0.5, legend.width=1, col=gray((0:255)/255), legend=FALSE,
    axes=FALSE, box=FALSE, xlab=NA, ylab=NA, xlim=c(xrmin,xrmax), ylim=c(yrmin,yrmax), useRaster=TRUE)
  par(new=TRUE)
}

#Plotting main raster layer
plot(layer, legend.width=1, useRaster=TRUE, col=coll, breaks=levl, xlim=c(xrmin,xrmax), ylim=c(yrmin,yrmax),
  axes=FALSE, xlab=NA, ylab=NA, box=FALSE, legend.args=list(text=titshort, side=3, line=1), axis.args=list(labels=lab))
par(new=TRUE)

#Plotting vectors
if ( impactdef==1 ) {

  par(new=TRUE)
  plot(impactshape, bg='transparent', col='green', lty=1, lwd=0.5, xlim=c(xsmin,xsmax), ylim=c(ysmin,ysmax),
    axes=FALSE, xlab=NA, ylab=NA)
}

if ( releasedef==1 ) {

  par(new=TRUE)
  plot(releaseshape, bg='transparent', col='green', lty=6, lwd=0.5, xlim=c(xsmin,xsmax), ylim=c(ysmin,ysmax),
    axes=FALSE, xlab=NA, ylab=NA)
}

if ( depositdef==1 ) {

  par(new=TRUE)
  plot(depositshape, bg='transparent', col='green', lty=5, lwd=0.5, xlim=c(xsmin,xsmax), ylim=c(ysmin,ysmax),
    axes=FALSE, xlab=NA, ylab=NA)
}

#Importing and drawing user-defined profile
if ( mapprof==1 ) {
  par(new=TRUE)
  intable<-paste(temppath,'/fmapprof.txt',sep='')
  profilex<-scan(intable, nlines=1)
  profiley<-scan(intable, skip=1, nlines=1)
  lines(x=profilex, y=profiley, col='yellow', lwd=1.5, lty=2)
}

#Plotting release points
if ( releasefile!='0' ) {

  if ( mode==1 || mode==2 ) { #with release hazard indicator score
    par(new=TRUE)
    points(pointdata$XR, pointdata$YR, pch=16, cex=min(max(500/nrows,0.2),1), col=pointdata$COLOUR)
    par(new=TRUE)
    points(pointdata$XR, pointdata$YR, cex=min(max(500/nrows,0.2),1), col='black', lwd=0.5)

  } else { #without release hazard indicator score
    par(new=TRUE)
    points(pointdata$XR, pointdata$YR, cex=min(max(250/nrows,0.2),1), col='black', lwd=0.5)
  }
}

#Plotting rectangle for text and legend
rect(xrmin-xdiff/2,yrmax,xrmax+xdiff/2,yrmax+ydiff/2, col='white', lty=0)
rect(xrmin-xdiff/2,yrmin-xdiff/2,xrmax+xdiff/2,yrmin, col='white', lty=0)

#Plotting title
text(x=xsmin+xdiff/2, y=ysmax+ydiff/12.5, labels=titlong, cex=1.25, col='black', font=2) #title

#Computing locations and labels of tick marks
xaxlbs<-vector() #initializing vector for x axis tick mark positions and labels
yaxlbs<-vector() #initializing vector for y axis tick mark positions and labels

ldiff<-100*round(xdiff/600, digits=0) #distance between tick marks
xlmin<-100*round((xsmin+ldiff/2)/100, digits=0) #position of first tick mark on x axis
ylmin<-100*round((ysmin+ldiff/2)/100, digits=0) #position of first tick mark on y axis

if ( ldiff==0 ) { #distance was set too large
  ldiff<-10*round(xdiff/60, digits=0) #distance between tick marks
  xlmin<-10*round((xsmin+ldiff/2)/10, digits=0) #position of first tick mark on x axis
  ylmin<-10*round((ysmin+ldiff/2)/10, digits=0) #position of first tick mark on y axis
}
j<-1 #initializing counter for loop over all x axis ticks and labels
repeat { #starting loop
    if ( xlmin+(j-1)*ldiff > xsmin ) xaxlbs[j]<-xlmin+(j-1)*ldiff #tick mark position
    if ( xlmin+(j-1)*ldiff > xsmax-ldiff ) break #break if last label was reached
    j<-j+1
}
j<-1 #initializing counter for loop over all y axis ticks and labels
repeat { #starting loop
  if ( ylmin+(j-1)*ldiff > ysmin ) yaxlbs[j]<-ylmin+(j-1)*ldiff #tick mark position
  if ( ylmin+(j-1)*ldiff > ysmax-ldiff ) break #break if last label was reached
  j<-j+1
}

#Plotting bounding box and axes
box()
par(cex=0.65)
axis(side=1, tck=-0.01, labels=NA, at=xaxlbs)
axis(side=2, tck=-0.01, labels=NA, at=yaxlbs)
axis(side=1, lwd=0, line=-0.6, at=xaxlbs)
axis(side=2, lwd=0, line=-0.6, at=yaxlbs)

#Plotting legends
par(cex=1.0)
par(xpd=TRUE)

if ( impactdef==1 ) {
  legend('bottom', lty=1, lwd=0.5, col='green', legend='Impact', text.col='black', bty='n', horiz=TRUE,
  x.intersp=0.25, inset=c(0.150,0.01))
}
if ( releasedef==1 ) {
  legend('bottomleft', lty=6, lwd=0.5, col='green', legend='Release', text.col='black', bty='n', horiz=TRUE,
  x.intersp=0.25, inset=c(0.035,0.01))
}
if ( depositdef==1 ) {
  legend('bottomright', lty=5, lwd=0.5, col='green', legend='Deposit', text.col='black', bty='n', horiz=TRUE,
  x.intersp=0.25, inset=c(0.035,0.01))
}

if ( releasefile!='0' ) {
  if (( mode==1 || mode==2 ) && mflag==0 ) {
    text(x=xsmin+xdiff/30, y=ysmin-ydiff/10.2, labels='RIS', cex=1, col='black')
    legend('bottomleft', pch=16, lty=NA, lwd=0.5, col=c(col6,col5,col4,col3,col2,col1,col0), pt.cex=min(max(500/nrows,0.2),1),
      legend=c('H','','','','','L',''), text.col='black', bty='n', horiz=TRUE, x.intersp=-0.4, text.width=xdiff/200, inset=c(0.11,0.01))
    legend('bottomleft', pch=1, lty=NA, lwd=0.5, col='black', pt.cex=min(max(500/nrows,0.2),1), legend=c('H','','','','','L',''),
      text.col='black', bty='n', horiz=TRUE, x.intersp=-0.4, text.width=xdiff/200, inset=c(0.11,0.01))
  } else {
    legend('bottomleft', pch=1, lty=NA, lwd=0.5, col='black', pt.cex=min(max(250/nrows,0.2),1), legend=c('Release points'),
      text.col='black', bty='n', horiz=TRUE, x.intersp=-0.4, inset=c(0.05,0.01))
  }
}

#Closing plot file
dev.off()
